
/*************************************************************************************************
This file is part of wxControlAsistencia

Copyright (C) 2009 Ezequiel Gastn Miravalles

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*************************************************************************************************/

/*************************************************************************************************
Software: wxControlAsistencia
Author: Ezequiel Gastn Miravalles
Website: http://www.neoegm.com/es/tech/programming/c-cpp/wxwidgets/wx-control-de-asistencia/
License: GNU GPL v3 (attached, read above)
*************************************************************************************************/

// *****************************************************************************
// Archivo: Lista.h
// Descripcin: Clase Lista
// Autor: Ezequiel Gastn Miravalles
// ltima modificacin: 28/07/2009
// Website: http://www.neoegm.com/
// *****************************************************************************

#pragma once

#include <iostream>

template <class TYPE>
class Lista
{
	struct Nodo
	{
		Nodo() { m_pNext = NULL; }

		TYPE m_tData;
		Nodo *m_pNext;
	};

	Nodo *m_pnHeader;

	//Para optimizacin
	int m_nCount;
	Nodo *m_pnLast;
	//-----------------

public:
	//Construccin y destruccin
	Lista() { Init(); }
	~Lista() { LiberarLista(); }
	
	Lista(const Lista& rlLista) { Init(); operator=(rlLista); }
	
	//Operador de asignacin
	Lista& operator=(const Lista& rlLista);

	//Informacin
	int Cantidad() { return m_nCount; }
	bool Vacia() { return m_nCount==0; }

	//Manejo de elementos
	int Agregar(TYPE& rtData);
	TYPE* Elem(int nPos);
	bool Eliminar(int nPos);
	void EliminarTodos();

	//Operadores
	TYPE& operator[](int nPos) { return *Elem(nPos); }			//Elem
	int operator<<(TYPE& rdData) { return Agregar(rdData); }	//Agregar

protected:
	void LiberarLista();
	void Init() { m_pnHeader = NULL; m_pnLast = NULL; m_nCount = 0; }
};

template <class TYPE>
Lista<TYPE>& Lista<TYPE>::operator=(const Lista<TYPE>& rlLista)
{
	EliminarTodos();
	
	//Copio toda la lista agregando elemento por elemento
	for (Nodo *pnTemp = rlLista.m_pnHeader; pnTemp; pnTemp = pnTemp->m_pNext)
		Agregar(pnTemp->m_tData);
	
	return *this;
}

template <class TYPE>
TYPE* Lista<TYPE>::Elem(int nPos)
{
	if (nPos >= m_nCount)
		return NULL;

	Nodo *pnNodo = m_pnHeader;

	for (int i = 0; i < nPos; i++)
		pnNodo = pnNodo?pnNodo->m_pNext:NULL;	//Control por las dudas

	return &pnNodo->m_tData;
}

template <class TYPE>
void Lista<TYPE>::LiberarLista()
{
	while (m_pnHeader)
	{
		Nodo *pnNext = m_pnHeader->m_pNext;
		if (m_pnHeader)
			delete m_pnHeader;
		m_pnHeader = pnNext;
	}
}

template <class TYPE>
int Lista<TYPE>::Agregar(TYPE& rtData)
{
	Nodo *pnNodo = new Nodo;

	if (!pnNodo)	//No se pudo alocar
		return -1;

	pnNodo->m_tData = rtData;

	if (m_pnLast)
		m_pnLast->m_pNext = pnNodo;
	else
		m_pnHeader = pnNodo;

	m_pnLast = pnNodo;

	return m_nCount++;
}

template <class TYPE>
bool Lista<TYPE>::Eliminar(int nPos)
{
	if (nPos >= m_nCount)
		return NULL;

	if (nPos == 0)		//Eliminar el primero
	{
		Nodo *pnTemp = m_pnHeader->m_pNext;

		if (m_pnHeader)
			delete m_pnHeader;
		m_pnHeader = pnTemp;

		m_nCount--;

		if (!m_nCount)
			m_pnLast = NULL;

		return true;
	}

	Nodo *pnNodo = m_pnHeader;

	for (int i = 0; i < nPos+1; i++)
	{
		if (i+1 == nPos)
		{
			if (!pnNodo->m_pNext)				//Verificacin de seguridad
				return false;

			Nodo *pnTemp = pnNodo->m_pNext->m_pNext;

			if (pnNodo->m_pNext)
				delete pnNodo->m_pNext;
			pnNodo->m_pNext = pnTemp;

			m_nCount--;

			if (i+2 == m_nCount)	//ltimo nodo
				m_pnLast = pnNodo;

			return true;
		}

		pnNodo = pnNodo?pnNodo->m_pNext:NULL;	//Verificacin de seguridad

		if (!pnNodo)							//Verificacin de seguridad
			return false;
	}

	return false;
}

template <class TYPE>
void Lista<TYPE>::EliminarTodos()
{
	LiberarLista();

	m_nCount = 0;
	m_pnLast = NULL;
}

template <class TYPE>
std::ostream& operator<<(std::ostream& oStream, Lista<TYPE>& rlLista)
{
	for (int i = 0; i < rlLista.Cantidad(); i++)
	{
		TYPE *tData = rlLista.Elem(i);

		if (tData)	//Por las dudas
		{
			oStream << *tData;		//Se asume que est definido
			oStream << std::endl;
		}
	}

	return oStream;
}
