
/*************************************************************************************************
This file is part of wxControlAsistencia

Copyright (C) 2009 Ezequiel Gastn Miravalles

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*************************************************************************************************/

/*************************************************************************************************
Software: wxControlAsistencia
Author: Ezequiel Gastn Miravalles
Website: http://www.neoegm.com/es/tech/programming/c-cpp/wxwidgets/wx-control-de-asistencia/
License: GNU GPL v3 (attached, read above)
*************************************************************************************************/

// *****************************************************************************
// Archivo: String.cpp
// Descripcin: Clase String (cdigo)
// Autor: Ezequiel Gastn Miravalles
// ltima modificacin: 08/08/2009
// Website: http://www.neoegm.com/
// *****************************************************************************

#include <iostream>

using std::ostream;
using std::istream;

#include "String.h"
#include <cstring>

//Consts
const int cnStringReadBufferSize = 16;
//------

String& String::operator=(const char *pszString)
{
	if (!pszString)
	{
		Vaciar();
		return *this;
	}

	int nLen = strlen(pszString);

	if (nLen < m_nAllocated)	//Si me alcanza el espacio
		strcpy(m_pszString, pszString);
	else
	{
		if (m_pszString)
		{
			delete [] m_pszString;
			m_pszString = NULL;
		}

		m_nAllocated = nLen+1;
		m_pszString = new char[m_nAllocated];
		
		if (!m_pszString)	//No se pudo reservar espacio
			m_nAllocated = 0;
		else
			strcpy(m_pszString, pszString);
	}

	return *this;
}

void String::Vaciar()
{
	if (m_pszString)
		delete [] m_pszString;

	m_pszString = NULL;
	m_nAllocated = 0;
}

String& String::operator+=(const char *pszString)
{
	int nLen = Largo() + strlen(pszString);

	if (nLen >= m_nAllocated)	//Se necesita realocar espacio
		if (!Realocar(nLen+1))
			return *this;

	strcat(m_pszString, pszString);

	return *this;
}

String String::operator+(String rsString)
{
	String sString(*this);

	sString += rsString;
	
	return sString;
}

bool String::Realocar(int nSize)
{
	char *pszOldStr = m_pszString;

	m_nAllocated = nSize;
	m_pszString = new char [nSize];

	if (!m_pszString)	//No se pudo reservar espacio
	{
		if (pszOldStr)
			delete [] pszOldStr;
		m_nAllocated = 0;
		return false;
	}

	if (pszOldStr)
	{
		strcpy(m_pszString, pszOldStr);
		delete [] pszOldStr;
	}
	else
		m_pszString[0] = '\0';

	return true;
}

ostream& operator<<(ostream& oStream, String& rsString)
{
	oStream << (const char*)rsString;
	return oStream;
}

istream& operator>>(istream& iStream, String& rsString)
{
	char pszBuffer[cnStringReadBufferSize];
	pszBuffer[0] = '\0';	//Por las dudas hacemos que sea una string vlida

	rsString.Vaciar();

	do
	{
		iStream.clear();
		iStream.getline(pszBuffer, cnStringReadBufferSize);
		rsString += pszBuffer;
	} while (iStream.fail() && (!iStream.eof()));

	return iStream;
}

